from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ImportOpenDataVersionMetaData")


@_attrs_define
class ImportOpenDataVersionMetaData:
    """
    Attributes:
        food (int):
        unit (int):
        category (int):
        property_ (int):
        store (int):
        conversion (int):
    """

    food: int
    unit: int
    category: int
    property_: int
    store: int
    conversion: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        food = self.food

        unit = self.unit

        category = self.category

        property_ = self.property_

        store = self.store

        conversion = self.conversion

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "food": food,
                "unit": unit,
                "category": category,
                "property": property_,
                "store": store,
                "conversion": conversion,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        food = d.pop("food")

        unit = d.pop("unit")

        category = d.pop("category")

        property_ = d.pop("property")

        store = d.pop("store")

        conversion = d.pop("conversion")

        import_open_data_version_meta_data = cls(
            food=food,
            unit=unit,
            category=category,
            property_=property_,
            store=store,
            conversion=conversion,
        )

        import_open_data_version_meta_data.additional_properties = d
        return import_open_data_version_meta_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.meal_plan import MealPlan
    from ..models.recipe_overview import RecipeOverview
    from ..models.user import User


T = TypeVar("T", bound="PatchedShoppingListRecipe")


@_attrs_define
class PatchedShoppingListRecipe:
    """
    Attributes:
        id (int | Unset):
        name (str | Unset):
        recipe (int | None | Unset):
        recipe_data (RecipeOverview | Unset): Adds nested create feature
        meal_plan_data (MealPlan | Unset): Adds nested create feature
        mealplan (int | None | Unset):
        servings (float | Unset):
        created_by (User | Unset): Adds nested create feature
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    recipe: int | None | Unset = UNSET
    recipe_data: RecipeOverview | Unset = UNSET
    meal_plan_data: MealPlan | Unset = UNSET
    mealplan: int | None | Unset = UNSET
    servings: float | Unset = UNSET
    created_by: User | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        recipe: int | None | Unset
        if isinstance(self.recipe, Unset):
            recipe = UNSET
        else:
            recipe = self.recipe

        recipe_data: dict[str, Any] | Unset = UNSET
        if not isinstance(self.recipe_data, Unset):
            recipe_data = self.recipe_data.to_dict()

        meal_plan_data: dict[str, Any] | Unset = UNSET
        if not isinstance(self.meal_plan_data, Unset):
            meal_plan_data = self.meal_plan_data.to_dict()

        mealplan: int | None | Unset
        if isinstance(self.mealplan, Unset):
            mealplan = UNSET
        else:
            mealplan = self.mealplan

        servings = self.servings

        created_by: dict[str, Any] | Unset = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if recipe is not UNSET:
            field_dict["recipe"] = recipe
        if recipe_data is not UNSET:
            field_dict["recipe_data"] = recipe_data
        if meal_plan_data is not UNSET:
            field_dict["meal_plan_data"] = meal_plan_data
        if mealplan is not UNSET:
            field_dict["mealplan"] = mealplan
        if servings is not UNSET:
            field_dict["servings"] = servings
        if created_by is not UNSET:
            field_dict["created_by"] = created_by

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.recipe, Unset):
            if isinstance(self.recipe, int):
                files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))
            else:
                files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))

        if not isinstance(self.recipe_data, Unset):
            files.append(("recipe_data", (None, json.dumps(self.recipe_data.to_dict()).encode(), "application/json")))

        if not isinstance(self.meal_plan_data, Unset):
            files.append(
                ("meal_plan_data", (None, json.dumps(self.meal_plan_data.to_dict()).encode(), "application/json"))
            )

        if not isinstance(self.mealplan, Unset):
            if isinstance(self.mealplan, int):
                files.append(("mealplan", (None, str(self.mealplan).encode(), "text/plain")))
            else:
                files.append(("mealplan", (None, str(self.mealplan).encode(), "text/plain")))

        if not isinstance(self.servings, Unset):
            files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.meal_plan import MealPlan
        from ..models.recipe_overview import RecipeOverview
        from ..models.user import User

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_recipe(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        recipe = _parse_recipe(d.pop("recipe", UNSET))

        _recipe_data = d.pop("recipe_data", UNSET)
        recipe_data: RecipeOverview | Unset
        if isinstance(_recipe_data, Unset):
            recipe_data = UNSET
        else:
            recipe_data = RecipeOverview.from_dict(_recipe_data)

        _meal_plan_data = d.pop("meal_plan_data", UNSET)
        meal_plan_data: MealPlan | Unset
        if isinstance(_meal_plan_data, Unset):
            meal_plan_data = UNSET
        else:
            meal_plan_data = MealPlan.from_dict(_meal_plan_data)

        def _parse_mealplan(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        mealplan = _parse_mealplan(d.pop("mealplan", UNSET))

        servings = d.pop("servings", UNSET)

        _created_by = d.pop("created_by", UNSET)
        created_by: User | Unset
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = User.from_dict(_created_by)

        patched_shopping_list_recipe = cls(
            id=id,
            name=name,
            recipe=recipe,
            recipe_data=recipe_data,
            meal_plan_data=meal_plan_data,
            mealplan=mealplan,
            servings=servings,
            created_by=created_by,
        )

        patched_shopping_list_recipe.additional_properties = d
        return patched_shopping_list_recipe

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="IngredientParserRequest")


@_attrs_define
class IngredientParserRequest:
    """
    Attributes:
        ingredient (str | Unset):
        ingredients (list[str] | Unset):
    """

    ingredient: str | Unset = UNSET
    ingredients: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ingredient = self.ingredient

        ingredients: list[str] | Unset = UNSET
        if not isinstance(self.ingredients, Unset):
            ingredients = self.ingredients

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ingredient is not UNSET:
            field_dict["ingredient"] = ingredient
        if ingredients is not UNSET:
            field_dict["ingredients"] = ingredients

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.ingredient, Unset):
            files.append(("ingredient", (None, str(self.ingredient).encode(), "text/plain")))

        if not isinstance(self.ingredients, Unset):
            for ingredients_item_element in self.ingredients:
                files.append(("ingredients", (None, str(ingredients_item_element).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ingredient = d.pop("ingredient", UNSET)

        ingredients = cast(list[str], d.pop("ingredients", UNSET))

        ingredient_parser_request = cls(
            ingredient=ingredient,
            ingredients=ingredients,
        )

        ingredient_parser_request.additional_properties = d
        return ingredient_parser_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ImportOpenDataResponseDetail")


@_attrs_define
class ImportOpenDataResponseDetail:
    """
    Attributes:
        total_created (int | Unset):  Default: 0.
        total_updated (int | Unset):  Default: 0.
        total_untouched (int | Unset):  Default: 0.
        total_errored (int | Unset):  Default: 0.
    """

    total_created: int | Unset = 0
    total_updated: int | Unset = 0
    total_untouched: int | Unset = 0
    total_errored: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_created = self.total_created

        total_updated = self.total_updated

        total_untouched = self.total_untouched

        total_errored = self.total_errored

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total_created is not UNSET:
            field_dict["total_created"] = total_created
        if total_updated is not UNSET:
            field_dict["total_updated"] = total_updated
        if total_untouched is not UNSET:
            field_dict["total_untouched"] = total_untouched
        if total_errored is not UNSET:
            field_dict["total_errored"] = total_errored

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        total_created = d.pop("total_created", UNSET)

        total_updated = d.pop("total_updated", UNSET)

        total_untouched = d.pop("total_untouched", UNSET)

        total_errored = d.pop("total_errored", UNSET)

        import_open_data_response_detail = cls(
            total_created=total_created,
            total_updated=total_updated,
            total_untouched=total_untouched,
            total_errored=total_errored,
        )

        import_open_data_response_detail.additional_properties = d
        return import_open_data_response_detail

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ApiPropertyTypeListCategoryItem(str, Enum):
    ALLERGEN = "ALLERGEN"
    GOAL = "GOAL"
    NUTRITION = "NUTRITION"
    OTHER = "OTHER"
    PRICE = "PRICE"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.keyword_label import KeywordLabel
    from ..models.user import User


T = TypeVar("T", bound="RecipeOverview")


@_attrs_define
class RecipeOverview:
    """Adds nested create feature

    Attributes:
        name (str):
        image (None | str):
        keywords (list[KeywordLabel]):
        working_time (int):
        waiting_time (int):
        created_by (User): Adds nested create feature
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        internal (bool):
        servings (int):
        servings_text (str):
        rating (float | None):
        last_cooked (datetime.datetime | None):
        new (bool):
        recent (str):
        id (int | Unset):
        description (None | str | Unset):
        private (bool | Unset):
    """

    name: str
    image: None | str
    keywords: list[KeywordLabel]
    working_time: int
    waiting_time: int
    created_by: User
    created_at: datetime.datetime
    updated_at: datetime.datetime
    internal: bool
    servings: int
    servings_text: str
    rating: float | None
    last_cooked: datetime.datetime | None
    new: bool
    recent: str
    id: int | Unset = UNSET
    description: None | str | Unset = UNSET
    private: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        image: None | str
        image = self.image

        keywords = []
        for keywords_item_data in self.keywords:
            keywords_item = keywords_item_data.to_dict()
            keywords.append(keywords_item)

        working_time = self.working_time

        waiting_time = self.waiting_time

        created_by = self.created_by.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        internal = self.internal

        servings = self.servings

        servings_text = self.servings_text

        rating: float | None
        rating = self.rating

        last_cooked: None | str
        if isinstance(self.last_cooked, datetime.datetime):
            last_cooked = self.last_cooked.isoformat()
        else:
            last_cooked = self.last_cooked

        new = self.new

        recent = self.recent

        id = self.id

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        private = self.private

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "image": image,
                "keywords": keywords,
                "working_time": working_time,
                "waiting_time": waiting_time,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "internal": internal,
                "servings": servings,
                "servings_text": servings_text,
                "rating": rating,
                "last_cooked": last_cooked,
                "new": new,
                "recent": recent,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if private is not UNSET:
            field_dict["private"] = private

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.keyword_label import KeywordLabel
        from ..models.user import User

        d = dict(src_dict)
        name = d.pop("name")

        def _parse_image(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        image = _parse_image(d.pop("image"))

        keywords = []
        _keywords = d.pop("keywords")
        for keywords_item_data in _keywords:
            keywords_item = KeywordLabel.from_dict(keywords_item_data)

            keywords.append(keywords_item)

        working_time = d.pop("working_time")

        waiting_time = d.pop("waiting_time")

        created_by = User.from_dict(d.pop("created_by"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        internal = d.pop("internal")

        servings = d.pop("servings")

        servings_text = d.pop("servings_text")

        def _parse_rating(data: object) -> float | None:
            if data is None:
                return data
            return cast(float | None, data)

        rating = _parse_rating(d.pop("rating"))

        def _parse_last_cooked(data: object) -> datetime.datetime | None:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_cooked_type_0 = isoparse(data)

                return last_cooked_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None, data)

        last_cooked = _parse_last_cooked(d.pop("last_cooked"))

        new = d.pop("new")

        recent = d.pop("recent")

        id = d.pop("id", UNSET)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        private = d.pop("private", UNSET)

        recipe_overview = cls(
            name=name,
            image=image,
            keywords=keywords,
            working_time=working_time,
            waiting_time=waiting_time,
            created_by=created_by,
            created_at=created_at,
            updated_at=updated_at,
            internal=internal,
            servings=servings,
            servings_text=servings_text,
            rating=rating,
            last_cooked=last_cooked,
            new=new,
            recent=recent,
            id=id,
            description=description,
            private=private,
        )

        recipe_overview.additional_properties = d
        return recipe_overview

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.property_type import PropertyType


T = TypeVar("T", bound="PatchedProperty")


@_attrs_define
class PatchedProperty:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            id (int | Unset):
            property_amount (float | None | Unset):
            property_type (PropertyType | Unset): Adds nested create feature
    """

    id: int | Unset = UNSET
    property_amount: float | None | Unset = UNSET
    property_type: PropertyType | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        property_amount: float | None | Unset
        if isinstance(self.property_amount, Unset):
            property_amount = UNSET
        else:
            property_amount = self.property_amount

        property_type: dict[str, Any] | Unset = UNSET
        if not isinstance(self.property_type, Unset):
            property_type = self.property_type.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if property_amount is not UNSET:
            field_dict["property_amount"] = property_amount
        if property_type is not UNSET:
            field_dict["property_type"] = property_type

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.property_amount, Unset):
            if isinstance(self.property_amount, float):
                files.append(("property_amount", (None, str(self.property_amount).encode(), "text/plain")))
            else:
                files.append(("property_amount", (None, str(self.property_amount).encode(), "text/plain")))

        if not isinstance(self.property_type, Unset):
            files.append(
                ("property_type", (None, json.dumps(self.property_type.to_dict()).encode(), "application/json"))
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.property_type import PropertyType

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        def _parse_property_amount(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        property_amount = _parse_property_amount(d.pop("property_amount", UNSET))

        _property_type = d.pop("property_type", UNSET)
        property_type: PropertyType | Unset
        if isinstance(_property_type, Unset):
            property_type = UNSET
        else:
            property_type = PropertyType.from_dict(_property_type)

        patched_property = cls(
            id=id,
            property_amount=property_amount,
            property_type=property_type,
        )

        patched_property.additional_properties = d
        return patched_property

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.shopping_list import ShoppingList
    from ..models.supermarket_category import SupermarketCategory


T = TypeVar("T", bound="FoodShopping")


@_attrs_define
class FoodShopping:
    """
    Attributes:
        name (str):
        supermarket_category (SupermarketCategory): Moves `UniqueValidator`'s from the validation stage to the save
            stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        shopping_lists (list[ShoppingList]):
        id (int | Unset):
        plural_name (None | str | Unset):
    """

    name: str
    supermarket_category: SupermarketCategory
    shopping_lists: list[ShoppingList]
    id: int | Unset = UNSET
    plural_name: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        supermarket_category = self.supermarket_category.to_dict()

        shopping_lists = []
        for shopping_lists_item_data in self.shopping_lists:
            shopping_lists_item = shopping_lists_item_data.to_dict()
            shopping_lists.append(shopping_lists_item)

        id = self.id

        plural_name: None | str | Unset
        if isinstance(self.plural_name, Unset):
            plural_name = UNSET
        else:
            plural_name = self.plural_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "supermarket_category": supermarket_category,
                "shopping_lists": shopping_lists,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if plural_name is not UNSET:
            field_dict["plural_name"] = plural_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.shopping_list import ShoppingList
        from ..models.supermarket_category import SupermarketCategory

        d = dict(src_dict)
        name = d.pop("name")

        supermarket_category = SupermarketCategory.from_dict(d.pop("supermarket_category"))

        shopping_lists = []
        _shopping_lists = d.pop("shopping_lists")
        for shopping_lists_item_data in _shopping_lists:
            shopping_lists_item = ShoppingList.from_dict(shopping_lists_item_data)

            shopping_lists.append(shopping_lists_item)

        id = d.pop("id", UNSET)

        def _parse_plural_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        plural_name = _parse_plural_name(d.pop("plural_name", UNSET))

        food_shopping = cls(
            name=name,
            supermarket_category=supermarket_category,
            shopping_lists=shopping_lists,
            id=id,
            plural_name=plural_name,
        )

        food_shopping.additional_properties = d
        return food_shopping

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeBatchUpdate")


@_attrs_define
class RecipeBatchUpdate:
    """
    Attributes:
        recipes (list[int]):
        keywords_add (list[int]):
        keywords_remove (list[int]):
        keywords_set (list[int]):
        shared_add (list[int]):
        shared_remove (list[int]):
        shared_set (list[int]):
        keywords_remove_all (bool | Unset):  Default: False.
        working_time (int | None | Unset):
        waiting_time (int | None | Unset):
        servings (int | None | Unset):
        servings_text (None | str | Unset):
        private (bool | None | Unset):
        shared_remove_all (bool | Unset):  Default: False.
        show_ingredient_overview (bool | None | Unset):
        clear_description (bool | None | Unset):
    """

    recipes: list[int]
    keywords_add: list[int]
    keywords_remove: list[int]
    keywords_set: list[int]
    shared_add: list[int]
    shared_remove: list[int]
    shared_set: list[int]
    keywords_remove_all: bool | Unset = False
    working_time: int | None | Unset = UNSET
    waiting_time: int | None | Unset = UNSET
    servings: int | None | Unset = UNSET
    servings_text: None | str | Unset = UNSET
    private: bool | None | Unset = UNSET
    shared_remove_all: bool | Unset = False
    show_ingredient_overview: bool | None | Unset = UNSET
    clear_description: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        recipes = self.recipes

        keywords_add = self.keywords_add

        keywords_remove = self.keywords_remove

        keywords_set = self.keywords_set

        shared_add = self.shared_add

        shared_remove = self.shared_remove

        shared_set = self.shared_set

        keywords_remove_all = self.keywords_remove_all

        working_time: int | None | Unset
        if isinstance(self.working_time, Unset):
            working_time = UNSET
        else:
            working_time = self.working_time

        waiting_time: int | None | Unset
        if isinstance(self.waiting_time, Unset):
            waiting_time = UNSET
        else:
            waiting_time = self.waiting_time

        servings: int | None | Unset
        if isinstance(self.servings, Unset):
            servings = UNSET
        else:
            servings = self.servings

        servings_text: None | str | Unset
        if isinstance(self.servings_text, Unset):
            servings_text = UNSET
        else:
            servings_text = self.servings_text

        private: bool | None | Unset
        if isinstance(self.private, Unset):
            private = UNSET
        else:
            private = self.private

        shared_remove_all = self.shared_remove_all

        show_ingredient_overview: bool | None | Unset
        if isinstance(self.show_ingredient_overview, Unset):
            show_ingredient_overview = UNSET
        else:
            show_ingredient_overview = self.show_ingredient_overview

        clear_description: bool | None | Unset
        if isinstance(self.clear_description, Unset):
            clear_description = UNSET
        else:
            clear_description = self.clear_description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "recipes": recipes,
                "keywords_add": keywords_add,
                "keywords_remove": keywords_remove,
                "keywords_set": keywords_set,
                "shared_add": shared_add,
                "shared_remove": shared_remove,
                "shared_set": shared_set,
            }
        )
        if keywords_remove_all is not UNSET:
            field_dict["keywords_remove_all"] = keywords_remove_all
        if working_time is not UNSET:
            field_dict["working_time"] = working_time
        if waiting_time is not UNSET:
            field_dict["waiting_time"] = waiting_time
        if servings is not UNSET:
            field_dict["servings"] = servings
        if servings_text is not UNSET:
            field_dict["servings_text"] = servings_text
        if private is not UNSET:
            field_dict["private"] = private
        if shared_remove_all is not UNSET:
            field_dict["shared_remove_all"] = shared_remove_all
        if show_ingredient_overview is not UNSET:
            field_dict["show_ingredient_overview"] = show_ingredient_overview
        if clear_description is not UNSET:
            field_dict["clear_description"] = clear_description

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for recipes_item_element in self.recipes:
            files.append(("recipes", (None, str(recipes_item_element).encode(), "text/plain")))

        for keywords_add_item_element in self.keywords_add:
            files.append(("keywords_add", (None, str(keywords_add_item_element).encode(), "text/plain")))

        for keywords_remove_item_element in self.keywords_remove:
            files.append(("keywords_remove", (None, str(keywords_remove_item_element).encode(), "text/plain")))

        for keywords_set_item_element in self.keywords_set:
            files.append(("keywords_set", (None, str(keywords_set_item_element).encode(), "text/plain")))

        for shared_add_item_element in self.shared_add:
            files.append(("shared_add", (None, str(shared_add_item_element).encode(), "text/plain")))

        for shared_remove_item_element in self.shared_remove:
            files.append(("shared_remove", (None, str(shared_remove_item_element).encode(), "text/plain")))

        for shared_set_item_element in self.shared_set:
            files.append(("shared_set", (None, str(shared_set_item_element).encode(), "text/plain")))

        if not isinstance(self.keywords_remove_all, Unset):
            files.append(("keywords_remove_all", (None, str(self.keywords_remove_all).encode(), "text/plain")))

        if not isinstance(self.working_time, Unset):
            if isinstance(self.working_time, int):
                files.append(("working_time", (None, str(self.working_time).encode(), "text/plain")))
            else:
                files.append(("working_time", (None, str(self.working_time).encode(), "text/plain")))

        if not isinstance(self.waiting_time, Unset):
            if isinstance(self.waiting_time, int):
                files.append(("waiting_time", (None, str(self.waiting_time).encode(), "text/plain")))
            else:
                files.append(("waiting_time", (None, str(self.waiting_time).encode(), "text/plain")))

        if not isinstance(self.servings, Unset):
            if isinstance(self.servings, int):
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))
            else:
                files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        if not isinstance(self.servings_text, Unset):
            if isinstance(self.servings_text, str):
                files.append(("servings_text", (None, str(self.servings_text).encode(), "text/plain")))
            else:
                files.append(("servings_text", (None, str(self.servings_text).encode(), "text/plain")))

        if not isinstance(self.private, Unset):
            if isinstance(self.private, bool):
                files.append(("private", (None, str(self.private).encode(), "text/plain")))
            else:
                files.append(("private", (None, str(self.private).encode(), "text/plain")))

        if not isinstance(self.shared_remove_all, Unset):
            files.append(("shared_remove_all", (None, str(self.shared_remove_all).encode(), "text/plain")))

        if not isinstance(self.show_ingredient_overview, Unset):
            if isinstance(self.show_ingredient_overview, bool):
                files.append(
                    ("show_ingredient_overview", (None, str(self.show_ingredient_overview).encode(), "text/plain"))
                )
            else:
                files.append(
                    ("show_ingredient_overview", (None, str(self.show_ingredient_overview).encode(), "text/plain"))
                )

        if not isinstance(self.clear_description, Unset):
            if isinstance(self.clear_description, bool):
                files.append(("clear_description", (None, str(self.clear_description).encode(), "text/plain")))
            else:
                files.append(("clear_description", (None, str(self.clear_description).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        recipes = cast(list[int], d.pop("recipes"))

        keywords_add = cast(list[int], d.pop("keywords_add"))

        keywords_remove = cast(list[int], d.pop("keywords_remove"))

        keywords_set = cast(list[int], d.pop("keywords_set"))

        shared_add = cast(list[int], d.pop("shared_add"))

        shared_remove = cast(list[int], d.pop("shared_remove"))

        shared_set = cast(list[int], d.pop("shared_set"))

        keywords_remove_all = d.pop("keywords_remove_all", UNSET)

        def _parse_working_time(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        working_time = _parse_working_time(d.pop("working_time", UNSET))

        def _parse_waiting_time(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        waiting_time = _parse_waiting_time(d.pop("waiting_time", UNSET))

        def _parse_servings(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        servings = _parse_servings(d.pop("servings", UNSET))

        def _parse_servings_text(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        servings_text = _parse_servings_text(d.pop("servings_text", UNSET))

        def _parse_private(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        private = _parse_private(d.pop("private", UNSET))

        shared_remove_all = d.pop("shared_remove_all", UNSET)

        def _parse_show_ingredient_overview(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        show_ingredient_overview = _parse_show_ingredient_overview(d.pop("show_ingredient_overview", UNSET))

        def _parse_clear_description(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        clear_description = _parse_clear_description(d.pop("clear_description", UNSET))

        recipe_batch_update = cls(
            recipes=recipes,
            keywords_add=keywords_add,
            keywords_remove=keywords_remove,
            keywords_set=keywords_set,
            shared_add=shared_add,
            shared_remove=shared_remove,
            shared_set=shared_set,
            keywords_remove_all=keywords_remove_all,
            working_time=working_time,
            waiting_time=waiting_time,
            servings=servings,
            servings_text=servings_text,
            private=private,
            shared_remove_all=shared_remove_all,
            show_ingredient_overview=show_ingredient_overview,
            clear_description=clear_description,
        )

        recipe_batch_update.additional_properties = d
        return recipe_batch_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecipeImport")


@_attrs_define
class RecipeImport:
    """
    Attributes:
        name (str):
        created_at (datetime.datetime):
        storage (int):
        space (int):
        id (int | Unset):
        file_uid (str | Unset):
        file_path (str | Unset):
    """

    name: str
    created_at: datetime.datetime
    storage: int
    space: int
    id: int | Unset = UNSET
    file_uid: str | Unset = UNSET
    file_path: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        created_at = self.created_at.isoformat()

        storage = self.storage

        space = self.space

        id = self.id

        file_uid = self.file_uid

        file_path = self.file_path

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "created_at": created_at,
                "storage": storage,
                "space": space,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if file_uid is not UNSET:
            field_dict["file_uid"] = file_uid
        if file_path is not UNSET:
            field_dict["file_path"] = file_path

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("storage", (None, str(self.storage).encode(), "text/plain")))

        files.append(("space", (None, str(self.space).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.file_uid, Unset):
            files.append(("file_uid", (None, str(self.file_uid).encode(), "text/plain")))

        if not isinstance(self.file_path, Unset):
            files.append(("file_path", (None, str(self.file_path).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        created_at = isoparse(d.pop("created_at"))

        storage = d.pop("storage")

        space = d.pop("space")

        id = d.pop("id", UNSET)

        file_uid = d.pop("file_uid", UNSET)

        file_path = d.pop("file_path", UNSET)

        recipe_import = cls(
            name=name,
            created_at=created_at,
            storage=storage,
            space=space,
            id=id,
            file_uid=file_uid,
            file_path=file_path,
        )

        recipe_import.additional_properties = d
        return recipe_import

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.keyword import Keyword


T = TypeVar("T", bound="ImportLog")


@_attrs_define
class ImportLog:
    """
    Attributes:
        type_ (str):
        keyword (Keyword): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        created_by (int):
        created_at (datetime.datetime):
        id (int | Unset):
        msg (str | Unset):
        running (bool | Unset):
        total_recipes (int | Unset):
        imported_recipes (int | Unset):
    """

    type_: str
    keyword: Keyword
    created_by: int
    created_at: datetime.datetime
    id: int | Unset = UNSET
    msg: str | Unset = UNSET
    running: bool | Unset = UNSET
    total_recipes: int | Unset = UNSET
    imported_recipes: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        keyword = self.keyword.to_dict()

        created_by = self.created_by

        created_at = self.created_at.isoformat()

        id = self.id

        msg = self.msg

        running = self.running

        total_recipes = self.total_recipes

        imported_recipes = self.imported_recipes

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "keyword": keyword,
                "created_by": created_by,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if msg is not UNSET:
            field_dict["msg"] = msg
        if running is not UNSET:
            field_dict["running"] = running
        if total_recipes is not UNSET:
            field_dict["total_recipes"] = total_recipes
        if imported_recipes is not UNSET:
            field_dict["imported_recipes"] = imported_recipes

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("type", (None, str(self.type_).encode(), "text/plain")))

        files.append(("keyword", (None, json.dumps(self.keyword.to_dict()).encode(), "application/json")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.msg, Unset):
            files.append(("msg", (None, str(self.msg).encode(), "text/plain")))

        if not isinstance(self.running, Unset):
            files.append(("running", (None, str(self.running).encode(), "text/plain")))

        if not isinstance(self.total_recipes, Unset):
            files.append(("total_recipes", (None, str(self.total_recipes).encode(), "text/plain")))

        if not isinstance(self.imported_recipes, Unset):
            files.append(("imported_recipes", (None, str(self.imported_recipes).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.keyword import Keyword

        d = dict(src_dict)
        type_ = d.pop("type")

        keyword = Keyword.from_dict(d.pop("keyword"))

        created_by = d.pop("created_by")

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        msg = d.pop("msg", UNSET)

        running = d.pop("running", UNSET)

        total_recipes = d.pop("total_recipes", UNSET)

        imported_recipes = d.pop("imported_recipes", UNSET)

        import_log = cls(
            type_=type_,
            keyword=keyword,
            created_by=created_by,
            created_at=created_at,
            id=id,
            msg=msg,
            running=running,
            total_recipes=total_recipes,
            imported_recipes=imported_recipes,
        )

        import_log.additional_properties = d
        return import_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..models.search_enum import SearchEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.search_fields import SearchFields
    from ..models.user import User


T = TypeVar("T", bound="PatchedSearchPreference")


@_attrs_define
class PatchedSearchPreference:
    """Adds nested create feature

    Attributes:
        user (User | Unset): Adds nested create feature
        search (SearchEnum | Unset): * `plain` - Simple
            * `phrase` - Phrase
            * `websearch` - Web
            * `raw` - Raw
        lookup (bool | Unset):
        unaccent (list[SearchFields] | None | Unset):
        icontains (list[SearchFields] | None | Unset):
        istartswith (list[SearchFields] | None | Unset):
        trigram (list[SearchFields] | None | Unset):
        fulltext (list[SearchFields] | None | Unset):
        trigram_threshold (float | Unset):
    """

    user: User | Unset = UNSET
    search: SearchEnum | Unset = UNSET
    lookup: bool | Unset = UNSET
    unaccent: list[SearchFields] | None | Unset = UNSET
    icontains: list[SearchFields] | None | Unset = UNSET
    istartswith: list[SearchFields] | None | Unset = UNSET
    trigram: list[SearchFields] | None | Unset = UNSET
    fulltext: list[SearchFields] | None | Unset = UNSET
    trigram_threshold: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user: dict[str, Any] | Unset = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        search: str | Unset = UNSET
        if not isinstance(self.search, Unset):
            search = self.search.value

        lookup = self.lookup

        unaccent: list[dict[str, Any]] | None | Unset
        if isinstance(self.unaccent, Unset):
            unaccent = UNSET
        elif isinstance(self.unaccent, list):
            unaccent = []
            for unaccent_type_0_item_data in self.unaccent:
                unaccent_type_0_item = unaccent_type_0_item_data.to_dict()
                unaccent.append(unaccent_type_0_item)

        else:
            unaccent = self.unaccent

        icontains: list[dict[str, Any]] | None | Unset
        if isinstance(self.icontains, Unset):
            icontains = UNSET
        elif isinstance(self.icontains, list):
            icontains = []
            for icontains_type_0_item_data in self.icontains:
                icontains_type_0_item = icontains_type_0_item_data.to_dict()
                icontains.append(icontains_type_0_item)

        else:
            icontains = self.icontains

        istartswith: list[dict[str, Any]] | None | Unset
        if isinstance(self.istartswith, Unset):
            istartswith = UNSET
        elif isinstance(self.istartswith, list):
            istartswith = []
            for istartswith_type_0_item_data in self.istartswith:
                istartswith_type_0_item = istartswith_type_0_item_data.to_dict()
                istartswith.append(istartswith_type_0_item)

        else:
            istartswith = self.istartswith

        trigram: list[dict[str, Any]] | None | Unset
        if isinstance(self.trigram, Unset):
            trigram = UNSET
        elif isinstance(self.trigram, list):
            trigram = []
            for trigram_type_0_item_data in self.trigram:
                trigram_type_0_item = trigram_type_0_item_data.to_dict()
                trigram.append(trigram_type_0_item)

        else:
            trigram = self.trigram

        fulltext: list[dict[str, Any]] | None | Unset
        if isinstance(self.fulltext, Unset):
            fulltext = UNSET
        elif isinstance(self.fulltext, list):
            fulltext = []
            for fulltext_type_0_item_data in self.fulltext:
                fulltext_type_0_item = fulltext_type_0_item_data.to_dict()
                fulltext.append(fulltext_type_0_item)

        else:
            fulltext = self.fulltext

        trigram_threshold = self.trigram_threshold

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if search is not UNSET:
            field_dict["search"] = search
        if lookup is not UNSET:
            field_dict["lookup"] = lookup
        if unaccent is not UNSET:
            field_dict["unaccent"] = unaccent
        if icontains is not UNSET:
            field_dict["icontains"] = icontains
        if istartswith is not UNSET:
            field_dict["istartswith"] = istartswith
        if trigram is not UNSET:
            field_dict["trigram"] = trigram
        if fulltext is not UNSET:
            field_dict["fulltext"] = fulltext
        if trigram_threshold is not UNSET:
            field_dict["trigram_threshold"] = trigram_threshold

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.user, Unset):
            files.append(("user", (None, json.dumps(self.user.to_dict()).encode(), "application/json")))

        if not isinstance(self.search, Unset):
            files.append(("search", (None, str(self.search.value).encode(), "text/plain")))

        if not isinstance(self.lookup, Unset):
            files.append(("lookup", (None, str(self.lookup).encode(), "text/plain")))

        if not isinstance(self.unaccent, Unset):
            if isinstance(self.unaccent, list):
                for unaccent_type_0_item_element in self.unaccent:
                    files.append(
                        (
                            "unaccent",
                            (None, json.dumps(unaccent_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("unaccent", (None, str(self.unaccent).encode(), "text/plain")))

        if not isinstance(self.icontains, Unset):
            if isinstance(self.icontains, list):
                for icontains_type_0_item_element in self.icontains:
                    files.append(
                        (
                            "icontains",
                            (None, json.dumps(icontains_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("icontains", (None, str(self.icontains).encode(), "text/plain")))

        if not isinstance(self.istartswith, Unset):
            if isinstance(self.istartswith, list):
                for istartswith_type_0_item_element in self.istartswith:
                    files.append(
                        (
                            "istartswith",
                            (None, json.dumps(istartswith_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("istartswith", (None, str(self.istartswith).encode(), "text/plain")))

        if not isinstance(self.trigram, Unset):
            if isinstance(self.trigram, list):
                for trigram_type_0_item_element in self.trigram:
                    files.append(
                        (
                            "trigram",
                            (None, json.dumps(trigram_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("trigram", (None, str(self.trigram).encode(), "text/plain")))

        if not isinstance(self.fulltext, Unset):
            if isinstance(self.fulltext, list):
                for fulltext_type_0_item_element in self.fulltext:
                    files.append(
                        (
                            "fulltext",
                            (None, json.dumps(fulltext_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("fulltext", (None, str(self.fulltext).encode(), "text/plain")))

        if not isinstance(self.trigram_threshold, Unset):
            files.append(("trigram_threshold", (None, str(self.trigram_threshold).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.search_fields import SearchFields
        from ..models.user import User

        d = dict(src_dict)
        _user = d.pop("user", UNSET)
        user: User | Unset
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = User.from_dict(_user)

        _search = d.pop("search", UNSET)
        search: SearchEnum | Unset
        if isinstance(_search, Unset):
            search = UNSET
        else:
            search = SearchEnum(_search)

        lookup = d.pop("lookup", UNSET)

        def _parse_unaccent(data: object) -> list[SearchFields] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                unaccent_type_0 = []
                _unaccent_type_0 = data
                for unaccent_type_0_item_data in _unaccent_type_0:
                    unaccent_type_0_item = SearchFields.from_dict(unaccent_type_0_item_data)

                    unaccent_type_0.append(unaccent_type_0_item)

                return unaccent_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[SearchFields] | None | Unset, data)

        unaccent = _parse_unaccent(d.pop("unaccent", UNSET))

        def _parse_icontains(data: object) -> list[SearchFields] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                icontains_type_0 = []
                _icontains_type_0 = data
                for icontains_type_0_item_data in _icontains_type_0:
                    icontains_type_0_item = SearchFields.from_dict(icontains_type_0_item_data)

                    icontains_type_0.append(icontains_type_0_item)

                return icontains_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[SearchFields] | None | Unset, data)

        icontains = _parse_icontains(d.pop("icontains", UNSET))

        def _parse_istartswith(data: object) -> list[SearchFields] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                istartswith_type_0 = []
                _istartswith_type_0 = data
                for istartswith_type_0_item_data in _istartswith_type_0:
                    istartswith_type_0_item = SearchFields.from_dict(istartswith_type_0_item_data)

                    istartswith_type_0.append(istartswith_type_0_item)

                return istartswith_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[SearchFields] | None | Unset, data)

        istartswith = _parse_istartswith(d.pop("istartswith", UNSET))

        def _parse_trigram(data: object) -> list[SearchFields] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                trigram_type_0 = []
                _trigram_type_0 = data
                for trigram_type_0_item_data in _trigram_type_0:
                    trigram_type_0_item = SearchFields.from_dict(trigram_type_0_item_data)

                    trigram_type_0.append(trigram_type_0_item)

                return trigram_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[SearchFields] | None | Unset, data)

        trigram = _parse_trigram(d.pop("trigram", UNSET))

        def _parse_fulltext(data: object) -> list[SearchFields] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                fulltext_type_0 = []
                _fulltext_type_0 = data
                for fulltext_type_0_item_data in _fulltext_type_0:
                    fulltext_type_0_item = SearchFields.from_dict(fulltext_type_0_item_data)

                    fulltext_type_0.append(fulltext_type_0_item)

                return fulltext_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[SearchFields] | None | Unset, data)

        fulltext = _parse_fulltext(d.pop("fulltext", UNSET))

        trigram_threshold = d.pop("trigram_threshold", UNSET)

        patched_search_preference = cls(
            user=user,
            search=search,
            lookup=lookup,
            unaccent=unaccent,
            icontains=icontains,
            istartswith=istartswith,
            trigram=trigram,
            fulltext=fulltext,
            trigram_threshold=trigram_threshold,
        )

        patched_search_preference.additional_properties = d
        return patched_search_preference

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.unit import Unit


T = TypeVar("T", bound="PaginatedUnitList")


@_attrs_define
class PaginatedUnitList:
    """
    Attributes:
        count (int):  Example: 123.
        results (list[Unit]):
        next_ (None | str | Unset):  Example: http://api.example.org/accounts/?page=4.
        previous (None | str | Unset):  Example: http://api.example.org/accounts/?page=2.
        timestamp (datetime.datetime | Unset):
    """

    count: int
    results: list[Unit]
    next_: None | str | Unset = UNSET
    previous: None | str | Unset = UNSET
    timestamp: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        count = self.count

        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()
            results.append(results_item)

        next_: None | str | Unset
        if isinstance(self.next_, Unset):
            next_ = UNSET
        else:
            next_ = self.next_

        previous: None | str | Unset
        if isinstance(self.previous, Unset):
            previous = UNSET
        else:
            previous = self.previous

        timestamp: str | Unset = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "count": count,
                "results": results,
            }
        )
        if next_ is not UNSET:
            field_dict["next"] = next_
        if previous is not UNSET:
            field_dict["previous"] = previous
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.unit import Unit

        d = dict(src_dict)
        count = d.pop("count")

        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = Unit.from_dict(results_item_data)

            results.append(results_item)

        def _parse_next_(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        next_ = _parse_next_(d.pop("next", UNSET))

        def _parse_previous(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        previous = _parse_previous(d.pop("previous", UNSET))

        _timestamp = d.pop("timestamp", UNSET)
        timestamp: datetime.datetime | Unset
        if isinstance(_timestamp, Unset):
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        paginated_unit_list = cls(
            count=count,
            results=results,
            next_=next_,
            previous=previous,
            timestamp=timestamp,
        )

        paginated_unit_list.additional_properties = d
        return paginated_unit_list

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.keyword import Keyword
    from ..models.nutrition_information import NutritionInformation
    from ..models.property_ import Property
    from ..models.step import Step
    from ..models.user import User


T = TypeVar("T", bound="PatchedRecipe")


@_attrs_define
class PatchedRecipe:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        description (None | str | Unset):
        image (None | str | Unset):
        keywords (list[Keyword] | Unset):
        steps (list[Step] | Unset):
        working_time (int | Unset):
        waiting_time (int | Unset):
        created_by (User | Unset): Adds nested create feature
        created_at (datetime.datetime | Unset):
        updated_at (datetime.datetime | Unset):
        source_url (None | str | Unset):
        internal (bool | Unset):
        show_ingredient_overview (bool | Unset):
        nutrition (None | NutritionInformation | Unset):
        properties (list[Property] | Unset):
        food_properties (Any | Unset):
        servings (int | Unset):
        file_path (str | Unset):
        servings_text (str | Unset):
        diameter (int | Unset):
        diameter_text (str | Unset):
        rating (float | None | Unset):
        last_cooked (datetime.datetime | None | Unset):
        private (bool | Unset):
        shared (list[User] | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    description: None | str | Unset = UNSET
    image: None | str | Unset = UNSET
    keywords: list[Keyword] | Unset = UNSET
    steps: list[Step] | Unset = UNSET
    working_time: int | Unset = UNSET
    waiting_time: int | Unset = UNSET
    created_by: User | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    updated_at: datetime.datetime | Unset = UNSET
    source_url: None | str | Unset = UNSET
    internal: bool | Unset = UNSET
    show_ingredient_overview: bool | Unset = UNSET
    nutrition: None | NutritionInformation | Unset = UNSET
    properties: list[Property] | Unset = UNSET
    food_properties: Any | Unset = UNSET
    servings: int | Unset = UNSET
    file_path: str | Unset = UNSET
    servings_text: str | Unset = UNSET
    diameter: int | Unset = UNSET
    diameter_text: str | Unset = UNSET
    rating: float | None | Unset = UNSET
    last_cooked: datetime.datetime | None | Unset = UNSET
    private: bool | Unset = UNSET
    shared: list[User] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.nutrition_information import NutritionInformation

        id = self.id

        name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        image: None | str | Unset
        if isinstance(self.image, Unset):
            image = UNSET
        else:
            image = self.image

        keywords: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.keywords, Unset):
            keywords = []
            for keywords_item_data in self.keywords:
                keywords_item = keywords_item_data.to_dict()
                keywords.append(keywords_item)

        steps: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.steps, Unset):
            steps = []
            for steps_item_data in self.steps:
                steps_item = steps_item_data.to_dict()
                steps.append(steps_item)

        working_time = self.working_time

        waiting_time = self.waiting_time

        created_by: dict[str, Any] | Unset = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        source_url: None | str | Unset
        if isinstance(self.source_url, Unset):
            source_url = UNSET
        else:
            source_url = self.source_url

        internal = self.internal

        show_ingredient_overview = self.show_ingredient_overview

        nutrition: dict[str, Any] | None | Unset
        if isinstance(self.nutrition, Unset):
            nutrition = UNSET
        elif isinstance(self.nutrition, NutritionInformation):
            nutrition = self.nutrition.to_dict()
        else:
            nutrition = self.nutrition

        properties: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        food_properties = self.food_properties

        servings = self.servings

        file_path = self.file_path

        servings_text = self.servings_text

        diameter = self.diameter

        diameter_text = self.diameter_text

        rating: float | None | Unset
        if isinstance(self.rating, Unset):
            rating = UNSET
        else:
            rating = self.rating

        last_cooked: None | str | Unset
        if isinstance(self.last_cooked, Unset):
            last_cooked = UNSET
        elif isinstance(self.last_cooked, datetime.datetime):
            last_cooked = self.last_cooked.isoformat()
        else:
            last_cooked = self.last_cooked

        private = self.private

        shared: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.shared, Unset):
            shared = []
            for shared_item_data in self.shared:
                shared_item = shared_item_data.to_dict()
                shared.append(shared_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if image is not UNSET:
            field_dict["image"] = image
        if keywords is not UNSET:
            field_dict["keywords"] = keywords
        if steps is not UNSET:
            field_dict["steps"] = steps
        if working_time is not UNSET:
            field_dict["working_time"] = working_time
        if waiting_time is not UNSET:
            field_dict["waiting_time"] = waiting_time
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if source_url is not UNSET:
            field_dict["source_url"] = source_url
        if internal is not UNSET:
            field_dict["internal"] = internal
        if show_ingredient_overview is not UNSET:
            field_dict["show_ingredient_overview"] = show_ingredient_overview
        if nutrition is not UNSET:
            field_dict["nutrition"] = nutrition
        if properties is not UNSET:
            field_dict["properties"] = properties
        if food_properties is not UNSET:
            field_dict["food_properties"] = food_properties
        if servings is not UNSET:
            field_dict["servings"] = servings
        if file_path is not UNSET:
            field_dict["file_path"] = file_path
        if servings_text is not UNSET:
            field_dict["servings_text"] = servings_text
        if diameter is not UNSET:
            field_dict["diameter"] = diameter
        if diameter_text is not UNSET:
            field_dict["diameter_text"] = diameter_text
        if rating is not UNSET:
            field_dict["rating"] = rating
        if last_cooked is not UNSET:
            field_dict["last_cooked"] = last_cooked
        if private is not UNSET:
            field_dict["private"] = private
        if shared is not UNSET:
            field_dict["shared"] = shared

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            if isinstance(self.description, str):
                files.append(("description", (None, str(self.description).encode(), "text/plain")))
            else:
                files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.image, Unset):
            if isinstance(self.image, str):
                files.append(("image", (None, str(self.image).encode(), "text/plain")))
            else:
                files.append(("image", (None, str(self.image).encode(), "text/plain")))

        if not isinstance(self.keywords, Unset):
            for keywords_item_element in self.keywords:
                files.append(
                    ("keywords", (None, json.dumps(keywords_item_element.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.steps, Unset):
            for steps_item_element in self.steps:
                files.append(("steps", (None, json.dumps(steps_item_element.to_dict()).encode(), "application/json")))

        if not isinstance(self.working_time, Unset):
            files.append(("working_time", (None, str(self.working_time).encode(), "text/plain")))

        if not isinstance(self.waiting_time, Unset):
            files.append(("waiting_time", (None, str(self.waiting_time).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.updated_at, Unset):
            files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.source_url, Unset):
            if isinstance(self.source_url, str):
                files.append(("source_url", (None, str(self.source_url).encode(), "text/plain")))
            else:
                files.append(("source_url", (None, str(self.source_url).encode(), "text/plain")))

        if not isinstance(self.internal, Unset):
            files.append(("internal", (None, str(self.internal).encode(), "text/plain")))

        if not isinstance(self.show_ingredient_overview, Unset):
            files.append(
                ("show_ingredient_overview", (None, str(self.show_ingredient_overview).encode(), "text/plain"))
            )

        if not isinstance(self.nutrition, Unset):
            if self.nutrition is None:
                files.append(("nutrition", (None, str(self.nutrition).encode(), "text/plain")))
            else:
                files.append(("nutrition", (None, json.dumps(self.nutrition.to_dict()).encode(), "application/json")))

        if not isinstance(self.properties, Unset):
            for properties_item_element in self.properties:
                files.append(
                    ("properties", (None, json.dumps(properties_item_element.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.food_properties, Unset):
            files.append(("food_properties", (None, str(self.food_properties).encode(), "text/plain")))

        if not isinstance(self.servings, Unset):
            files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        if not isinstance(self.file_path, Unset):
            files.append(("file_path", (None, str(self.file_path).encode(), "text/plain")))

        if not isinstance(self.servings_text, Unset):
            files.append(("servings_text", (None, str(self.servings_text).encode(), "text/plain")))

        if not isinstance(self.diameter, Unset):
            files.append(("diameter", (None, str(self.diameter).encode(), "text/plain")))

        if not isinstance(self.diameter_text, Unset):
            files.append(("diameter_text", (None, str(self.diameter_text).encode(), "text/plain")))

        if not isinstance(self.rating, Unset):
            if isinstance(self.rating, float):
                files.append(("rating", (None, str(self.rating).encode(), "text/plain")))
            else:
                files.append(("rating", (None, str(self.rating).encode(), "text/plain")))

        if not isinstance(self.last_cooked, Unset):
            if isinstance(self.last_cooked, datetime.datetime):
                files.append(("last_cooked", (None, self.last_cooked.isoformat().encode(), "text/plain")))
            else:
                files.append(("last_cooked", (None, str(self.last_cooked).encode(), "text/plain")))

        if not isinstance(self.private, Unset):
            files.append(("private", (None, str(self.private).encode(), "text/plain")))

        if not isinstance(self.shared, Unset):
            for shared_item_element in self.shared:
                files.append(("shared", (None, json.dumps(shared_item_element.to_dict()).encode(), "application/json")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.keyword import Keyword
        from ..models.nutrition_information import NutritionInformation
        from ..models.property_ import Property
        from ..models.step import Step
        from ..models.user import User

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_image(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        image = _parse_image(d.pop("image", UNSET))

        _keywords = d.pop("keywords", UNSET)
        keywords: list[Keyword] | Unset = UNSET
        if _keywords is not UNSET:
            keywords = []
            for keywords_item_data in _keywords:
                keywords_item = Keyword.from_dict(keywords_item_data)

                keywords.append(keywords_item)

        _steps = d.pop("steps", UNSET)
        steps: list[Step] | Unset = UNSET
        if _steps is not UNSET:
            steps = []
            for steps_item_data in _steps:
                steps_item = Step.from_dict(steps_item_data)

                steps.append(steps_item)

        working_time = d.pop("working_time", UNSET)

        waiting_time = d.pop("waiting_time", UNSET)

        _created_by = d.pop("created_by", UNSET)
        created_by: User | Unset
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = User.from_dict(_created_by)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        def _parse_source_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        source_url = _parse_source_url(d.pop("source_url", UNSET))

        internal = d.pop("internal", UNSET)

        show_ingredient_overview = d.pop("show_ingredient_overview", UNSET)

        def _parse_nutrition(data: object) -> None | NutritionInformation | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                nutrition_type_1 = NutritionInformation.from_dict(data)

                return nutrition_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | NutritionInformation | Unset, data)

        nutrition = _parse_nutrition(d.pop("nutrition", UNSET))

        _properties = d.pop("properties", UNSET)
        properties: list[Property] | Unset = UNSET
        if _properties is not UNSET:
            properties = []
            for properties_item_data in _properties:
                properties_item = Property.from_dict(properties_item_data)

                properties.append(properties_item)

        food_properties = d.pop("food_properties", UNSET)

        servings = d.pop("servings", UNSET)

        file_path = d.pop("file_path", UNSET)

        servings_text = d.pop("servings_text", UNSET)

        diameter = d.pop("diameter", UNSET)

        diameter_text = d.pop("diameter_text", UNSET)

        def _parse_rating(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        rating = _parse_rating(d.pop("rating", UNSET))

        def _parse_last_cooked(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_cooked_type_0 = isoparse(data)

                return last_cooked_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_cooked = _parse_last_cooked(d.pop("last_cooked", UNSET))

        private = d.pop("private", UNSET)

        _shared = d.pop("shared", UNSET)
        shared: list[User] | Unset = UNSET
        if _shared is not UNSET:
            shared = []
            for shared_item_data in _shared:
                shared_item = User.from_dict(shared_item_data)

                shared.append(shared_item)

        patched_recipe = cls(
            id=id,
            name=name,
            description=description,
            image=image,
            keywords=keywords,
            steps=steps,
            working_time=working_time,
            waiting_time=waiting_time,
            created_by=created_by,
            created_at=created_at,
            updated_at=updated_at,
            source_url=source_url,
            internal=internal,
            show_ingredient_overview=show_ingredient_overview,
            nutrition=nutrition,
            properties=properties,
            food_properties=food_properties,
            servings=servings,
            file_path=file_path,
            servings_text=servings_text,
            diameter=diameter,
            diameter_text=diameter_text,
            rating=rating,
            last_cooked=last_cooked,
            private=private,
            shared=shared,
        )

        patched_recipe.additional_properties = d
        return patched_recipe

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..models.delete_enum import DeleteEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="FoodShoppingUpdate")


@_attrs_define
class FoodShoppingUpdate:
    """
    Attributes:
        delete (DeleteEnum | None): When set to true will delete all food from active shopping lists.

            * `true` - true
        id (int | Unset):
        amount (int | None | Unset): Amount of food to add to the shopping list
        unit (int | None | Unset): ID of unit to use for the shopping list
    """

    delete: DeleteEnum | None
    id: int | Unset = UNSET
    amount: int | None | Unset = UNSET
    unit: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        delete: None | str
        if isinstance(self.delete, DeleteEnum):
            delete = self.delete.value
        else:
            delete = self.delete

        id = self.id

        amount: int | None | Unset
        if isinstance(self.amount, Unset):
            amount = UNSET
        else:
            amount = self.amount

        unit: int | None | Unset
        if isinstance(self.unit, Unset):
            unit = UNSET
        else:
            unit = self.unit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "delete": delete,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if amount is not UNSET:
            field_dict["amount"] = amount
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if self.delete is None:
            files.append(("delete", (None, str(self.delete).encode(), "text/plain")))
        else:
            files.append(("delete", (None, str(self.delete.value).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.amount, Unset):
            if isinstance(self.amount, int):
                files.append(("amount", (None, str(self.amount).encode(), "text/plain")))
            else:
                files.append(("amount", (None, str(self.amount).encode(), "text/plain")))

        if not isinstance(self.unit, Unset):
            if isinstance(self.unit, int):
                files.append(("unit", (None, str(self.unit).encode(), "text/plain")))
            else:
                files.append(("unit", (None, str(self.unit).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_delete(data: object) -> DeleteEnum | None:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                delete_type_1 = DeleteEnum(data)

                return delete_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DeleteEnum | None, data)

        delete = _parse_delete(d.pop("delete"))

        id = d.pop("id", UNSET)

        def _parse_amount(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        amount = _parse_amount(d.pop("amount", UNSET))

        def _parse_unit(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        unit = _parse_unit(d.pop("unit", UNSET))

        food_shopping_update = cls(
            delete=delete,
            id=id,
            amount=amount,
            unit=unit,
        )

        food_shopping_update.additional_properties = d
        return food_shopping_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.shopping_list import ShoppingList
    from ..models.supermarket_category_relation import SupermarketCategoryRelation


T = TypeVar("T", bound="PatchedSupermarket")


@_attrs_define
class PatchedSupermarket:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            id (int | Unset):
            name (str | Unset):
            description (None | str | Unset):
            shopping_lists (list[ShoppingList] | Unset):
            category_to_supermarket (list[SupermarketCategoryRelation] | Unset):
            open_data_slug (None | str | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    description: None | str | Unset = UNSET
    shopping_lists: list[ShoppingList] | Unset = UNSET
    category_to_supermarket: list[SupermarketCategoryRelation] | Unset = UNSET
    open_data_slug: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        shopping_lists: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.shopping_lists, Unset):
            shopping_lists = []
            for shopping_lists_item_data in self.shopping_lists:
                shopping_lists_item = shopping_lists_item_data.to_dict()
                shopping_lists.append(shopping_lists_item)

        category_to_supermarket: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.category_to_supermarket, Unset):
            category_to_supermarket = []
            for category_to_supermarket_item_data in self.category_to_supermarket:
                category_to_supermarket_item = category_to_supermarket_item_data.to_dict()
                category_to_supermarket.append(category_to_supermarket_item)

        open_data_slug: None | str | Unset
        if isinstance(self.open_data_slug, Unset):
            open_data_slug = UNSET
        else:
            open_data_slug = self.open_data_slug

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if shopping_lists is not UNSET:
            field_dict["shopping_lists"] = shopping_lists
        if category_to_supermarket is not UNSET:
            field_dict["category_to_supermarket"] = category_to_supermarket
        if open_data_slug is not UNSET:
            field_dict["open_data_slug"] = open_data_slug

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            if isinstance(self.description, str):
                files.append(("description", (None, str(self.description).encode(), "text/plain")))
            else:
                files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.shopping_lists, Unset):
            for shopping_lists_item_element in self.shopping_lists:
                files.append(
                    (
                        "shopping_lists",
                        (None, json.dumps(shopping_lists_item_element.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.category_to_supermarket, Unset):
            for category_to_supermarket_item_element in self.category_to_supermarket:
                files.append(
                    (
                        "category_to_supermarket",
                        (None, json.dumps(category_to_supermarket_item_element.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.open_data_slug, Unset):
            if isinstance(self.open_data_slug, str):
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))
            else:
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.shopping_list import ShoppingList
        from ..models.supermarket_category_relation import SupermarketCategoryRelation

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        _shopping_lists = d.pop("shopping_lists", UNSET)
        shopping_lists: list[ShoppingList] | Unset = UNSET
        if _shopping_lists is not UNSET:
            shopping_lists = []
            for shopping_lists_item_data in _shopping_lists:
                shopping_lists_item = ShoppingList.from_dict(shopping_lists_item_data)

                shopping_lists.append(shopping_lists_item)

        _category_to_supermarket = d.pop("category_to_supermarket", UNSET)
        category_to_supermarket: list[SupermarketCategoryRelation] | Unset = UNSET
        if _category_to_supermarket is not UNSET:
            category_to_supermarket = []
            for category_to_supermarket_item_data in _category_to_supermarket:
                category_to_supermarket_item = SupermarketCategoryRelation.from_dict(category_to_supermarket_item_data)

                category_to_supermarket.append(category_to_supermarket_item)

        def _parse_open_data_slug(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        open_data_slug = _parse_open_data_slug(d.pop("open_data_slug", UNSET))

        patched_supermarket = cls(
            id=id,
            name=name,
            description=description,
            shopping_lists=shopping_lists,
            category_to_supermarket=category_to_supermarket,
            open_data_slug=open_data_slug,
        )

        patched_supermarket.additional_properties = d
        return patched_supermarket

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food_shopping import FoodShopping
    from ..models.shopping_list import ShoppingList
    from ..models.shopping_list_recipe import ShoppingListRecipe
    from ..models.unit import Unit
    from ..models.user import User


T = TypeVar("T", bound="ShoppingListEntry")


@_attrs_define
class ShoppingListEntry:
    """Adds nested create feature

    Attributes:
        food (FoodShopping | None):
        amount (float):
        list_recipe_data (ShoppingListRecipe):
        created_by (User): Adds nested create feature
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        id (int | Unset):
        list_recipe (int | None | Unset):
        shopping_lists (list[ShoppingList] | Unset):
        unit (None | Unit | Unset):
        order (int | Unset):
        checked (bool | Unset):
        ingredient (int | None | Unset):
        completed_at (datetime.datetime | None | Unset):
        delay_until (datetime.datetime | None | Unset):
        mealplan_id (int | Unset): If a mealplan id is given try to find existing or create new ShoppingListRecipe with
            that meal plan and link entry to it
    """

    food: FoodShopping | None
    amount: float
    list_recipe_data: ShoppingListRecipe
    created_by: User
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    list_recipe: int | None | Unset = UNSET
    shopping_lists: list[ShoppingList] | Unset = UNSET
    unit: None | Unit | Unset = UNSET
    order: int | Unset = UNSET
    checked: bool | Unset = UNSET
    ingredient: int | None | Unset = UNSET
    completed_at: datetime.datetime | None | Unset = UNSET
    delay_until: datetime.datetime | None | Unset = UNSET
    mealplan_id: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.food_shopping import FoodShopping
        from ..models.unit import Unit

        food: dict[str, Any] | None
        if isinstance(self.food, FoodShopping):
            food = self.food.to_dict()
        else:
            food = self.food

        amount = self.amount

        list_recipe_data = self.list_recipe_data.to_dict()

        created_by = self.created_by.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        list_recipe: int | None | Unset
        if isinstance(self.list_recipe, Unset):
            list_recipe = UNSET
        else:
            list_recipe = self.list_recipe

        shopping_lists: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.shopping_lists, Unset):
            shopping_lists = []
            for shopping_lists_item_data in self.shopping_lists:
                shopping_lists_item = shopping_lists_item_data.to_dict()
                shopping_lists.append(shopping_lists_item)

        unit: dict[str, Any] | None | Unset
        if isinstance(self.unit, Unset):
            unit = UNSET
        elif isinstance(self.unit, Unit):
            unit = self.unit.to_dict()
        else:
            unit = self.unit

        order = self.order

        checked = self.checked

        ingredient: int | None | Unset
        if isinstance(self.ingredient, Unset):
            ingredient = UNSET
        else:
            ingredient = self.ingredient

        completed_at: None | str | Unset
        if isinstance(self.completed_at, Unset):
            completed_at = UNSET
        elif isinstance(self.completed_at, datetime.datetime):
            completed_at = self.completed_at.isoformat()
        else:
            completed_at = self.completed_at

        delay_until: None | str | Unset
        if isinstance(self.delay_until, Unset):
            delay_until = UNSET
        elif isinstance(self.delay_until, datetime.datetime):
            delay_until = self.delay_until.isoformat()
        else:
            delay_until = self.delay_until

        mealplan_id = self.mealplan_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "food": food,
                "amount": amount,
                "list_recipe_data": list_recipe_data,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if list_recipe is not UNSET:
            field_dict["list_recipe"] = list_recipe
        if shopping_lists is not UNSET:
            field_dict["shopping_lists"] = shopping_lists
        if unit is not UNSET:
            field_dict["unit"] = unit
        if order is not UNSET:
            field_dict["order"] = order
        if checked is not UNSET:
            field_dict["checked"] = checked
        if ingredient is not UNSET:
            field_dict["ingredient"] = ingredient
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if delay_until is not UNSET:
            field_dict["delay_until"] = delay_until
        if mealplan_id is not UNSET:
            field_dict["mealplan_id"] = mealplan_id

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if self.food is None:
            files.append(("food", (None, str(self.food).encode(), "text/plain")))
        else:
            files.append(("food", (None, json.dumps(self.food.to_dict()).encode(), "application/json")))

        files.append(("amount", (None, str(self.amount).encode(), "text/plain")))

        files.append(
            ("list_recipe_data", (None, json.dumps(self.list_recipe_data.to_dict()).encode(), "application/json"))
        )

        files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.list_recipe, Unset):
            if isinstance(self.list_recipe, int):
                files.append(("list_recipe", (None, str(self.list_recipe).encode(), "text/plain")))
            else:
                files.append(("list_recipe", (None, str(self.list_recipe).encode(), "text/plain")))

        if not isinstance(self.shopping_lists, Unset):
            for shopping_lists_item_element in self.shopping_lists:
                files.append(
                    (
                        "shopping_lists",
                        (None, json.dumps(shopping_lists_item_element.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.unit, Unset):
            if self.unit is None:
                files.append(("unit", (None, str(self.unit).encode(), "text/plain")))
            else:
                files.append(("unit", (None, json.dumps(self.unit.to_dict()).encode(), "application/json")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        if not isinstance(self.checked, Unset):
            files.append(("checked", (None, str(self.checked).encode(), "text/plain")))

        if not isinstance(self.ingredient, Unset):
            if isinstance(self.ingredient, int):
                files.append(("ingredient", (None, str(self.ingredient).encode(), "text/plain")))
            else:
                files.append(("ingredient", (None, str(self.ingredient).encode(), "text/plain")))

        if not isinstance(self.completed_at, Unset):
            if isinstance(self.completed_at, datetime.datetime):
                files.append(("completed_at", (None, self.completed_at.isoformat().encode(), "text/plain")))
            else:
                files.append(("completed_at", (None, str(self.completed_at).encode(), "text/plain")))

        if not isinstance(self.delay_until, Unset):
            if isinstance(self.delay_until, datetime.datetime):
                files.append(("delay_until", (None, self.delay_until.isoformat().encode(), "text/plain")))
            else:
                files.append(("delay_until", (None, str(self.delay_until).encode(), "text/plain")))

        if not isinstance(self.mealplan_id, Unset):
            files.append(("mealplan_id", (None, str(self.mealplan_id).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food_shopping import FoodShopping
        from ..models.shopping_list import ShoppingList
        from ..models.shopping_list_recipe import ShoppingListRecipe
        from ..models.unit import Unit
        from ..models.user import User

        d = dict(src_dict)

        def _parse_food(data: object) -> FoodShopping | None:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                food_type_1 = FoodShopping.from_dict(data)

                return food_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FoodShopping | None, data)

        food = _parse_food(d.pop("food"))

        amount = d.pop("amount")

        list_recipe_data = ShoppingListRecipe.from_dict(d.pop("list_recipe_data"))

        created_by = User.from_dict(d.pop("created_by"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        def _parse_list_recipe(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        list_recipe = _parse_list_recipe(d.pop("list_recipe", UNSET))

        _shopping_lists = d.pop("shopping_lists", UNSET)
        shopping_lists: list[ShoppingList] | Unset = UNSET
        if _shopping_lists is not UNSET:
            shopping_lists = []
            for shopping_lists_item_data in _shopping_lists:
                shopping_lists_item = ShoppingList.from_dict(shopping_lists_item_data)

                shopping_lists.append(shopping_lists_item)

        def _parse_unit(data: object) -> None | Unit | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                unit_type_1 = Unit.from_dict(data)

                return unit_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unit | Unset, data)

        unit = _parse_unit(d.pop("unit", UNSET))

        order = d.pop("order", UNSET)

        checked = d.pop("checked", UNSET)

        def _parse_ingredient(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        ingredient = _parse_ingredient(d.pop("ingredient", UNSET))

        def _parse_completed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                completed_at_type_0 = isoparse(data)

                return completed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        completed_at = _parse_completed_at(d.pop("completed_at", UNSET))

        def _parse_delay_until(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                delay_until_type_0 = isoparse(data)

                return delay_until_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        delay_until = _parse_delay_until(d.pop("delay_until", UNSET))

        mealplan_id = d.pop("mealplan_id", UNSET)

        shopping_list_entry = cls(
            food=food,
            amount=amount,
            list_recipe_data=list_recipe_data,
            created_by=created_by,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            list_recipe=list_recipe,
            shopping_lists=shopping_lists,
            unit=unit,
            order=order,
            checked=checked,
            ingredient=ingredient,
            completed_at=completed_at,
            delay_until=delay_until,
            mealplan_id=mealplan_id,
        )

        shopping_list_entry.additional_properties = d
        return shopping_list_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..models.space_nav_text_color_enum import SpaceNavTextColorEnum
from ..models.space_theme_enum import SpaceThemeEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ai_provider import AiProvider
    from ..models.food_inherit_field import FoodInheritField
    from ..models.user import User
    from ..models.user_file_view import UserFileView


T = TypeVar("T", bound="PatchedSpace")


@_attrs_define
class PatchedSpace:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        created_by (User | Unset): Adds nested create feature
        created_at (datetime.datetime | Unset):
        message (str | Unset):
        max_recipes (int | Unset):
        max_file_storage_mb (int | Unset): Maximum file storage for space in MB. 0 for unlimited, -1 to disable file
            upload.
        max_users (int | Unset):
        allow_sharing (bool | Unset):
        demo (bool | Unset):
        food_inherit (list[FoodInheritField] | Unset):
        user_count (int | Unset):
        recipe_count (int | Unset):
        file_size_mb (float | Unset):
        image (None | Unset | UserFileView):
        nav_logo (None | Unset | UserFileView):
        space_theme (SpaceThemeEnum | Unset): * `BLANK` - -------
            * `TANDOOR` - Tandoor
            * `BOOTSTRAP` - Bootstrap
            * `DARKLY` - Darkly
            * `FLATLY` - Flatly
            * `SUPERHERO` - Superhero
            * `TANDOOR_DARK` - Tandoor Dark (INCOMPLETE)
        custom_space_theme (None | Unset | UserFileView):
        nav_bg_color (str | Unset):
        nav_text_color (SpaceNavTextColorEnum | Unset): * `BLANK` - -------
            * `LIGHT` - Light
            * `DARK` - Dark
        logo_color_32 (None | Unset | UserFileView):
        logo_color_128 (None | Unset | UserFileView):
        logo_color_144 (None | Unset | UserFileView):
        logo_color_180 (None | Unset | UserFileView):
        logo_color_192 (None | Unset | UserFileView):
        logo_color_512 (None | Unset | UserFileView):
        logo_color_svg (None | Unset | UserFileView):
        ai_credits_monthly (int | Unset):
        ai_credits_balance (float | Unset):
        ai_monthly_credits_used (int | Unset):
        ai_enabled (bool | Unset):
        ai_default_provider (AiProvider | None | Unset):
        space_setup_completed (bool | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    created_by: User | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    message: str | Unset = UNSET
    max_recipes: int | Unset = UNSET
    max_file_storage_mb: int | Unset = UNSET
    max_users: int | Unset = UNSET
    allow_sharing: bool | Unset = UNSET
    demo: bool | Unset = UNSET
    food_inherit: list[FoodInheritField] | Unset = UNSET
    user_count: int | Unset = UNSET
    recipe_count: int | Unset = UNSET
    file_size_mb: float | Unset = UNSET
    image: None | Unset | UserFileView = UNSET
    nav_logo: None | Unset | UserFileView = UNSET
    space_theme: SpaceThemeEnum | Unset = UNSET
    custom_space_theme: None | Unset | UserFileView = UNSET
    nav_bg_color: str | Unset = UNSET
    nav_text_color: SpaceNavTextColorEnum | Unset = UNSET
    logo_color_32: None | Unset | UserFileView = UNSET
    logo_color_128: None | Unset | UserFileView = UNSET
    logo_color_144: None | Unset | UserFileView = UNSET
    logo_color_180: None | Unset | UserFileView = UNSET
    logo_color_192: None | Unset | UserFileView = UNSET
    logo_color_512: None | Unset | UserFileView = UNSET
    logo_color_svg: None | Unset | UserFileView = UNSET
    ai_credits_monthly: int | Unset = UNSET
    ai_credits_balance: float | Unset = UNSET
    ai_monthly_credits_used: int | Unset = UNSET
    ai_enabled: bool | Unset = UNSET
    ai_default_provider: AiProvider | None | Unset = UNSET
    space_setup_completed: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.ai_provider import AiProvider
        from ..models.user_file_view import UserFileView

        id = self.id

        name = self.name

        created_by: dict[str, Any] | Unset = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        message = self.message

        max_recipes = self.max_recipes

        max_file_storage_mb = self.max_file_storage_mb

        max_users = self.max_users

        allow_sharing = self.allow_sharing

        demo = self.demo

        food_inherit: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.food_inherit, Unset):
            food_inherit = []
            for food_inherit_item_data in self.food_inherit:
                food_inherit_item = food_inherit_item_data.to_dict()
                food_inherit.append(food_inherit_item)

        user_count = self.user_count

        recipe_count = self.recipe_count

        file_size_mb = self.file_size_mb

        image: dict[str, Any] | None | Unset
        if isinstance(self.image, Unset):
            image = UNSET
        elif isinstance(self.image, UserFileView):
            image = self.image.to_dict()
        else:
            image = self.image

        nav_logo: dict[str, Any] | None | Unset
        if isinstance(self.nav_logo, Unset):
            nav_logo = UNSET
        elif isinstance(self.nav_logo, UserFileView):
            nav_logo = self.nav_logo.to_dict()
        else:
            nav_logo = self.nav_logo

        space_theme: str | Unset = UNSET
        if not isinstance(self.space_theme, Unset):
            space_theme = self.space_theme.value

        custom_space_theme: dict[str, Any] | None | Unset
        if isinstance(self.custom_space_theme, Unset):
            custom_space_theme = UNSET
        elif isinstance(self.custom_space_theme, UserFileView):
            custom_space_theme = self.custom_space_theme.to_dict()
        else:
            custom_space_theme = self.custom_space_theme

        nav_bg_color = self.nav_bg_color

        nav_text_color: str | Unset = UNSET
        if not isinstance(self.nav_text_color, Unset):
            nav_text_color = self.nav_text_color.value

        logo_color_32: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_32, Unset):
            logo_color_32 = UNSET
        elif isinstance(self.logo_color_32, UserFileView):
            logo_color_32 = self.logo_color_32.to_dict()
        else:
            logo_color_32 = self.logo_color_32

        logo_color_128: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_128, Unset):
            logo_color_128 = UNSET
        elif isinstance(self.logo_color_128, UserFileView):
            logo_color_128 = self.logo_color_128.to_dict()
        else:
            logo_color_128 = self.logo_color_128

        logo_color_144: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_144, Unset):
            logo_color_144 = UNSET
        elif isinstance(self.logo_color_144, UserFileView):
            logo_color_144 = self.logo_color_144.to_dict()
        else:
            logo_color_144 = self.logo_color_144

        logo_color_180: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_180, Unset):
            logo_color_180 = UNSET
        elif isinstance(self.logo_color_180, UserFileView):
            logo_color_180 = self.logo_color_180.to_dict()
        else:
            logo_color_180 = self.logo_color_180

        logo_color_192: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_192, Unset):
            logo_color_192 = UNSET
        elif isinstance(self.logo_color_192, UserFileView):
            logo_color_192 = self.logo_color_192.to_dict()
        else:
            logo_color_192 = self.logo_color_192

        logo_color_512: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_512, Unset):
            logo_color_512 = UNSET
        elif isinstance(self.logo_color_512, UserFileView):
            logo_color_512 = self.logo_color_512.to_dict()
        else:
            logo_color_512 = self.logo_color_512

        logo_color_svg: dict[str, Any] | None | Unset
        if isinstance(self.logo_color_svg, Unset):
            logo_color_svg = UNSET
        elif isinstance(self.logo_color_svg, UserFileView):
            logo_color_svg = self.logo_color_svg.to_dict()
        else:
            logo_color_svg = self.logo_color_svg

        ai_credits_monthly = self.ai_credits_monthly

        ai_credits_balance = self.ai_credits_balance

        ai_monthly_credits_used = self.ai_monthly_credits_used

        ai_enabled = self.ai_enabled

        ai_default_provider: dict[str, Any] | None | Unset
        if isinstance(self.ai_default_provider, Unset):
            ai_default_provider = UNSET
        elif isinstance(self.ai_default_provider, AiProvider):
            ai_default_provider = self.ai_default_provider.to_dict()
        else:
            ai_default_provider = self.ai_default_provider

        space_setup_completed = self.space_setup_completed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if message is not UNSET:
            field_dict["message"] = message
        if max_recipes is not UNSET:
            field_dict["max_recipes"] = max_recipes
        if max_file_storage_mb is not UNSET:
            field_dict["max_file_storage_mb"] = max_file_storage_mb
        if max_users is not UNSET:
            field_dict["max_users"] = max_users
        if allow_sharing is not UNSET:
            field_dict["allow_sharing"] = allow_sharing
        if demo is not UNSET:
            field_dict["demo"] = demo
        if food_inherit is not UNSET:
            field_dict["food_inherit"] = food_inherit
        if user_count is not UNSET:
            field_dict["user_count"] = user_count
        if recipe_count is not UNSET:
            field_dict["recipe_count"] = recipe_count
        if file_size_mb is not UNSET:
            field_dict["file_size_mb"] = file_size_mb
        if image is not UNSET:
            field_dict["image"] = image
        if nav_logo is not UNSET:
            field_dict["nav_logo"] = nav_logo
        if space_theme is not UNSET:
            field_dict["space_theme"] = space_theme
        if custom_space_theme is not UNSET:
            field_dict["custom_space_theme"] = custom_space_theme
        if nav_bg_color is not UNSET:
            field_dict["nav_bg_color"] = nav_bg_color
        if nav_text_color is not UNSET:
            field_dict["nav_text_color"] = nav_text_color
        if logo_color_32 is not UNSET:
            field_dict["logo_color_32"] = logo_color_32
        if logo_color_128 is not UNSET:
            field_dict["logo_color_128"] = logo_color_128
        if logo_color_144 is not UNSET:
            field_dict["logo_color_144"] = logo_color_144
        if logo_color_180 is not UNSET:
            field_dict["logo_color_180"] = logo_color_180
        if logo_color_192 is not UNSET:
            field_dict["logo_color_192"] = logo_color_192
        if logo_color_512 is not UNSET:
            field_dict["logo_color_512"] = logo_color_512
        if logo_color_svg is not UNSET:
            field_dict["logo_color_svg"] = logo_color_svg
        if ai_credits_monthly is not UNSET:
            field_dict["ai_credits_monthly"] = ai_credits_monthly
        if ai_credits_balance is not UNSET:
            field_dict["ai_credits_balance"] = ai_credits_balance
        if ai_monthly_credits_used is not UNSET:
            field_dict["ai_monthly_credits_used"] = ai_monthly_credits_used
        if ai_enabled is not UNSET:
            field_dict["ai_enabled"] = ai_enabled
        if ai_default_provider is not UNSET:
            field_dict["ai_default_provider"] = ai_default_provider
        if space_setup_completed is not UNSET:
            field_dict["space_setup_completed"] = space_setup_completed

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        if not isinstance(self.created_at, Unset):
            files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.message, Unset):
            files.append(("message", (None, str(self.message).encode(), "text/plain")))

        if not isinstance(self.max_recipes, Unset):
            files.append(("max_recipes", (None, str(self.max_recipes).encode(), "text/plain")))

        if not isinstance(self.max_file_storage_mb, Unset):
            files.append(("max_file_storage_mb", (None, str(self.max_file_storage_mb).encode(), "text/plain")))

        if not isinstance(self.max_users, Unset):
            files.append(("max_users", (None, str(self.max_users).encode(), "text/plain")))

        if not isinstance(self.allow_sharing, Unset):
            files.append(("allow_sharing", (None, str(self.allow_sharing).encode(), "text/plain")))

        if not isinstance(self.demo, Unset):
            files.append(("demo", (None, str(self.demo).encode(), "text/plain")))

        if not isinstance(self.food_inherit, Unset):
            for food_inherit_item_element in self.food_inherit:
                files.append(
                    (
                        "food_inherit",
                        (None, json.dumps(food_inherit_item_element.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.user_count, Unset):
            files.append(("user_count", (None, str(self.user_count).encode(), "text/plain")))

        if not isinstance(self.recipe_count, Unset):
            files.append(("recipe_count", (None, str(self.recipe_count).encode(), "text/plain")))

        if not isinstance(self.file_size_mb, Unset):
            files.append(("file_size_mb", (None, str(self.file_size_mb).encode(), "text/plain")))

        if not isinstance(self.image, Unset):
            if self.image is None:
                files.append(("image", (None, str(self.image).encode(), "text/plain")))
            else:
                files.append(("image", (None, json.dumps(self.image.to_dict()).encode(), "application/json")))

        if not isinstance(self.nav_logo, Unset):
            if self.nav_logo is None:
                files.append(("nav_logo", (None, str(self.nav_logo).encode(), "text/plain")))
            else:
                files.append(("nav_logo", (None, json.dumps(self.nav_logo.to_dict()).encode(), "application/json")))

        if not isinstance(self.space_theme, Unset):
            files.append(("space_theme", (None, str(self.space_theme.value).encode(), "text/plain")))

        if not isinstance(self.custom_space_theme, Unset):
            if self.custom_space_theme is None:
                files.append(("custom_space_theme", (None, str(self.custom_space_theme).encode(), "text/plain")))
            else:
                files.append(
                    (
                        "custom_space_theme",
                        (None, json.dumps(self.custom_space_theme.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.nav_bg_color, Unset):
            files.append(("nav_bg_color", (None, str(self.nav_bg_color).encode(), "text/plain")))

        if not isinstance(self.nav_text_color, Unset):
            files.append(("nav_text_color", (None, str(self.nav_text_color.value).encode(), "text/plain")))

        if not isinstance(self.logo_color_32, Unset):
            if self.logo_color_32 is None:
                files.append(("logo_color_32", (None, str(self.logo_color_32).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_32", (None, json.dumps(self.logo_color_32.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_128, Unset):
            if self.logo_color_128 is None:
                files.append(("logo_color_128", (None, str(self.logo_color_128).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_128", (None, json.dumps(self.logo_color_128.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_144, Unset):
            if self.logo_color_144 is None:
                files.append(("logo_color_144", (None, str(self.logo_color_144).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_144", (None, json.dumps(self.logo_color_144.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_180, Unset):
            if self.logo_color_180 is None:
                files.append(("logo_color_180", (None, str(self.logo_color_180).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_180", (None, json.dumps(self.logo_color_180.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_192, Unset):
            if self.logo_color_192 is None:
                files.append(("logo_color_192", (None, str(self.logo_color_192).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_192", (None, json.dumps(self.logo_color_192.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_512, Unset):
            if self.logo_color_512 is None:
                files.append(("logo_color_512", (None, str(self.logo_color_512).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_512", (None, json.dumps(self.logo_color_512.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.logo_color_svg, Unset):
            if self.logo_color_svg is None:
                files.append(("logo_color_svg", (None, str(self.logo_color_svg).encode(), "text/plain")))
            else:
                files.append(
                    ("logo_color_svg", (None, json.dumps(self.logo_color_svg.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.ai_credits_monthly, Unset):
            files.append(("ai_credits_monthly", (None, str(self.ai_credits_monthly).encode(), "text/plain")))

        if not isinstance(self.ai_credits_balance, Unset):
            files.append(("ai_credits_balance", (None, str(self.ai_credits_balance).encode(), "text/plain")))

        if not isinstance(self.ai_monthly_credits_used, Unset):
            files.append(("ai_monthly_credits_used", (None, str(self.ai_monthly_credits_used).encode(), "text/plain")))

        if not isinstance(self.ai_enabled, Unset):
            files.append(("ai_enabled", (None, str(self.ai_enabled).encode(), "text/plain")))

        if not isinstance(self.ai_default_provider, Unset):
            if self.ai_default_provider is None:
                files.append(("ai_default_provider", (None, str(self.ai_default_provider).encode(), "text/plain")))
            else:
                files.append(
                    (
                        "ai_default_provider",
                        (None, json.dumps(self.ai_default_provider.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.space_setup_completed, Unset):
            files.append(("space_setup_completed", (None, str(self.space_setup_completed).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ai_provider import AiProvider
        from ..models.food_inherit_field import FoodInheritField
        from ..models.user import User
        from ..models.user_file_view import UserFileView

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _created_by = d.pop("created_by", UNSET)
        created_by: User | Unset
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = User.from_dict(_created_by)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        message = d.pop("message", UNSET)

        max_recipes = d.pop("max_recipes", UNSET)

        max_file_storage_mb = d.pop("max_file_storage_mb", UNSET)

        max_users = d.pop("max_users", UNSET)

        allow_sharing = d.pop("allow_sharing", UNSET)

        demo = d.pop("demo", UNSET)

        _food_inherit = d.pop("food_inherit", UNSET)
        food_inherit: list[FoodInheritField] | Unset = UNSET
        if _food_inherit is not UNSET:
            food_inherit = []
            for food_inherit_item_data in _food_inherit:
                food_inherit_item = FoodInheritField.from_dict(food_inherit_item_data)

                food_inherit.append(food_inherit_item)

        user_count = d.pop("user_count", UNSET)

        recipe_count = d.pop("recipe_count", UNSET)

        file_size_mb = d.pop("file_size_mb", UNSET)

        def _parse_image(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                image_type_1 = UserFileView.from_dict(data)

                return image_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        image = _parse_image(d.pop("image", UNSET))

        def _parse_nav_logo(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                nav_logo_type_1 = UserFileView.from_dict(data)

                return nav_logo_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        nav_logo = _parse_nav_logo(d.pop("nav_logo", UNSET))

        _space_theme = d.pop("space_theme", UNSET)
        space_theme: SpaceThemeEnum | Unset
        if isinstance(_space_theme, Unset):
            space_theme = UNSET
        else:
            space_theme = SpaceThemeEnum(_space_theme)

        def _parse_custom_space_theme(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                custom_space_theme_type_1 = UserFileView.from_dict(data)

                return custom_space_theme_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        custom_space_theme = _parse_custom_space_theme(d.pop("custom_space_theme", UNSET))

        nav_bg_color = d.pop("nav_bg_color", UNSET)

        _nav_text_color = d.pop("nav_text_color", UNSET)
        nav_text_color: SpaceNavTextColorEnum | Unset
        if isinstance(_nav_text_color, Unset):
            nav_text_color = UNSET
        else:
            nav_text_color = SpaceNavTextColorEnum(_nav_text_color)

        def _parse_logo_color_32(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_32_type_1 = UserFileView.from_dict(data)

                return logo_color_32_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_32 = _parse_logo_color_32(d.pop("logo_color_32", UNSET))

        def _parse_logo_color_128(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_128_type_1 = UserFileView.from_dict(data)

                return logo_color_128_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_128 = _parse_logo_color_128(d.pop("logo_color_128", UNSET))

        def _parse_logo_color_144(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_144_type_1 = UserFileView.from_dict(data)

                return logo_color_144_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_144 = _parse_logo_color_144(d.pop("logo_color_144", UNSET))

        def _parse_logo_color_180(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_180_type_1 = UserFileView.from_dict(data)

                return logo_color_180_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_180 = _parse_logo_color_180(d.pop("logo_color_180", UNSET))

        def _parse_logo_color_192(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_192_type_1 = UserFileView.from_dict(data)

                return logo_color_192_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_192 = _parse_logo_color_192(d.pop("logo_color_192", UNSET))

        def _parse_logo_color_512(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_512_type_1 = UserFileView.from_dict(data)

                return logo_color_512_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_512 = _parse_logo_color_512(d.pop("logo_color_512", UNSET))

        def _parse_logo_color_svg(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logo_color_svg_type_1 = UserFileView.from_dict(data)

                return logo_color_svg_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        logo_color_svg = _parse_logo_color_svg(d.pop("logo_color_svg", UNSET))

        ai_credits_monthly = d.pop("ai_credits_monthly", UNSET)

        ai_credits_balance = d.pop("ai_credits_balance", UNSET)

        ai_monthly_credits_used = d.pop("ai_monthly_credits_used", UNSET)

        ai_enabled = d.pop("ai_enabled", UNSET)

        def _parse_ai_default_provider(data: object) -> AiProvider | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                ai_default_provider_type_1 = AiProvider.from_dict(data)

                return ai_default_provider_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AiProvider | None | Unset, data)

        ai_default_provider = _parse_ai_default_provider(d.pop("ai_default_provider", UNSET))

        space_setup_completed = d.pop("space_setup_completed", UNSET)

        patched_space = cls(
            id=id,
            name=name,
            created_by=created_by,
            created_at=created_at,
            message=message,
            max_recipes=max_recipes,
            max_file_storage_mb=max_file_storage_mb,
            max_users=max_users,
            allow_sharing=allow_sharing,
            demo=demo,
            food_inherit=food_inherit,
            user_count=user_count,
            recipe_count=recipe_count,
            file_size_mb=file_size_mb,
            image=image,
            nav_logo=nav_logo,
            space_theme=space_theme,
            custom_space_theme=custom_space_theme,
            nav_bg_color=nav_bg_color,
            nav_text_color=nav_text_color,
            logo_color_32=logo_color_32,
            logo_color_128=logo_color_128,
            logo_color_144=logo_color_144,
            logo_color_180=logo_color_180,
            logo_color_192=logo_color_192,
            logo_color_512=logo_color_512,
            logo_color_svg=logo_color_svg,
            ai_credits_monthly=ai_credits_monthly,
            ai_credits_balance=ai_credits_balance,
            ai_monthly_credits_used=ai_monthly_credits_used,
            ai_enabled=ai_enabled,
            ai_default_provider=ai_default_provider,
            space_setup_completed=space_setup_completed,
        )

        patched_space.additional_properties = d
        return patched_space

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DefaultPageEnum(str, Enum):
    BOOKS = "BOOKS"
    PLAN = "PLAN"
    SEARCH = "SEARCH"
    SHOPPING = "SHOPPING"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="AiProvider")


@_attrs_define
class AiProvider:
    """
    Attributes:
        name (str):
        model_name (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        id (int | Unset):
        description (str | Unset):
        api_key (str | Unset):
        url (None | str | Unset):
        log_credit_cost (bool | Unset):
        space (int | None | Unset):
    """

    name: str
    model_name: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: int | Unset = UNSET
    description: str | Unset = UNSET
    api_key: str | Unset = UNSET
    url: None | str | Unset = UNSET
    log_credit_cost: bool | Unset = UNSET
    space: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        model_name = self.model_name

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        description = self.description

        api_key = self.api_key

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        log_credit_cost = self.log_credit_cost

        space: int | None | Unset
        if isinstance(self.space, Unset):
            space = UNSET
        else:
            space = self.space

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "model_name": model_name,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if api_key is not UNSET:
            field_dict["api_key"] = api_key
        if url is not UNSET:
            field_dict["url"] = url
        if log_credit_cost is not UNSET:
            field_dict["log_credit_cost"] = log_credit_cost
        if space is not UNSET:
            field_dict["space"] = space

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("model_name", (None, str(self.model_name).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        files.append(("updated_at", (None, self.updated_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.api_key, Unset):
            files.append(("api_key", (None, str(self.api_key).encode(), "text/plain")))

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.log_credit_cost, Unset):
            files.append(("log_credit_cost", (None, str(self.log_credit_cost).encode(), "text/plain")))

        if not isinstance(self.space, Unset):
            if isinstance(self.space, int):
                files.append(("space", (None, str(self.space).encode(), "text/plain")))
            else:
                files.append(("space", (None, str(self.space).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        model_name = d.pop("model_name")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id", UNSET)

        description = d.pop("description", UNSET)

        api_key = d.pop("api_key", UNSET)

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        log_credit_cost = d.pop("log_credit_cost", UNSET)

        def _parse_space(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        space = _parse_space(d.pop("space", UNSET))

        ai_provider = cls(
            name=name,
            model_name=model_name,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            description=description,
            api_key=api_key,
            url=url,
            log_credit_cost=log_credit_cost,
            space=space,
        )

        ai_provider.additional_properties = d
        return ai_provider

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
